local assets =
{
Asset("ANIM", "anim/crafting_table.zip"),
Asset("ATLAS", "images/inventoryimages/altar_broken_portable.xml"),
Asset("IMAGE", "images/inventoryimages/altar_broken_portable.tex"),
}

local function turnlightoff(inst, light)
	inst.SoundEmitter:KillSound("idlesound")
	if light then
		light:Enable(false)
	end
end

-- light, rad, intensity, falloff, colour, time, callback
local function OnTurnOn(inst)
	inst.components.prototyper.on = true  -- prototyper doesn't set this until after this function is called!!
	--ShowState(inst)
	inst.SoundEmitter:PlaySound("dontstarve/common/ancienttable_LP","idlesound")
	
	inst.Light:Enable(true)
	inst.components.lighttweener:StartTween(nil, 1.5, nil, nil, nil, 0.5) 
end
local function OnTurnOff(inst)
	inst.components.prototyper.on = false  -- prototyper doesn't set this until after this function is called
	--ShowState(inst)
	inst.components.lighttweener:StartTween(nil, 0, nil, nil, nil, 1, turnlightoff) 
end

local function ondrop(inst)
	inst:AddTag("prototyper")
	inst:AddTag("altar")
	inst:AddTag("structure")
	inst:AddTag("stone")
	inst:AddComponent("prototyper")
	inst.components.prototyper.onturnon = OnTurnOn
	inst.components.prototyper.onturnoff = OnTurnOff
	inst.components.prototyper.trees = TUNING.PROTOTYPER_TREES.ANCIENTALTAR_LOW
	inst.components.prototyper.onactivate = function()
		inst.AnimState:PlayAnimation("use")
		inst.AnimState:PushAnimation("idle_full")
		inst.AnimState:PushAnimation("proximity_loop", true)
		--[[inst.SoundEmitter:PlaySound("dontstarve/common/ancienttable_craft","sound")

		inst:DoTaskInTime(1.5, function() 
			inst.SoundEmitter:KillSound("sound")
			inst.SoundEmitter:PlaySound("dontstarve/common/researchmachine_"..soundprefix.."_ding","sound")	
		end)--]]
	end
	inst:AddComponent("lighttweener")
	local light = inst.entity:AddLight()
	inst.Light:Enable(false)
	inst.Light:SetRadius(.6)
	inst.Light:SetFalloff(1)
	inst.Light:SetIntensity(.5)
	inst.Light:SetColour(1,1,1)
	inst.components.lighttweener:StartTween(light, 1, .9, .9, {255/255,255/255,255/255}, 0, turnlightoff)
end

local function onpickup(inst)
	inst:RemoveTag("prototyper")
	inst:RemoveTag("altar")
	inst:RemoveTag("structure")
	inst:RemoveTag("stone")
	inst:RemoveComponent("prototyper")
	inst:RemoveComponent("lighttweener")
end

local function fn(Sim)

	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddPhysics()
    MakeInventoryPhysics(inst)
	
    inst.AnimState:SetBank("crafting_table")
    inst.AnimState:SetBuild("crafting_table")
    inst.AnimState:PlayAnimation("idle_broken")
	trans:SetScale(.4,.4,.4)
		
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "altar_broken_portable.tex" )

	--local minimap = inst.entity:AddMiniMapEntity()
	--minimap:SetPriority( 5 )
	--minimap:SetIcon( "tab_crafting_table.png" )
	
	inst:AddComponent("edible")
    inst.components.edible.foodtype = "ELEMENTAL"
    inst.components.edible.hungervalue = 2
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/altar_broken_portable.xml"
	inst.components.inventoryitem:SetOnDroppedFn(ondrop)
	inst.components.inventoryitem:SetOnPickupFn(onpickup)
	
	return inst
end

return Prefab( "common/inventory/altar_broken_portable", fn, assets) 
